'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let lemmatize = (() => {
  var _ref = _asyncToGenerator(function* (sentence) {
    const result = yield processWithCoreNLP(sentence);
    const coreNLPToken = result.document.sentences.sentence.tokens.token;
    const tokens = Array.isArray(coreNLPToken) ? coreNLPToken : [coreNLPToken];
    const lemmatized = tokens.map(function (token, i) {
      let shouldSpaced = true;
      let lemma;

      switch (token.POS) {
        case 'CD':
          // cardinal numbers have no lemma.
          // (TreeTagger put '@CARD@' as a lemma of CD)
          lemma = token.word;
          break;

        case 'PRP':
          // Replace personal pronouns with abstract ones.
          if (/sel(?:f|ves)$/.test(token.word)) {
            lemma = 'oneself';
          } else {
            lemma = 'one';
          }
          break;

        case 'PRP$':
          // Replace possessive pronoun with abstract one.
          lemma = 'one\'s';
          break;

        case 'POS':
          // Always use possessive ending "'s" even for plural nouns
          // to enable QuizGenerator process easily.
          lemma = '\'s';
          break;

        case 'MD':
          // Stanford POS Tagger returns the lemma of `'ll` is `'ll`,
          // which should be `'will`.
          if (token.word === "'ll") {
            lemma = 'will';
          } else {
            lemma = token.lemma;
          }
          break;

        default:
          if (tokens[i + 1] && tokens[i + 1].POS === 'POS') {
            // Replace all possessive proper nouns with "one's".
            lemma = 'one';
          } else {
            lemma = token.lemma || token.word;
          }
      }

      // For contraction, make sure the lemma starts an apostrophe and
      // a space is not inserted before it
      // so that both word counting and word splitting will not be affected by the lemmatizing.
      // i.e. making no difference between an original and lemmatized texts.
      const isContraction = token.word.startsWith("'") || token.word === "n't";
      if (isContraction) {
        if (!lemma.startsWith("'")) {
          lemma = `'${ lemma }`;
        }
        shouldSpaced = false;
      }

      // Not spaced before the beginning of a sentence.
      if (i === 0) {
        shouldSpaced = false;
      }

      return shouldSpaced ? ` ${ lemma }` : lemma;
    }).join('');

    return lemmatized;
  });

  return function lemmatize(_x) {
    return _ref.apply(this, arguments);
  };
})();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _stanfordCorenlp = require('stanford-corenlp');

var _stanfordCorenlp2 = _interopRequireDefault(_stanfordCorenlp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable no-console */


// Installing stanford-corenlp is not necessary if you won't use this module,
// so it is not listed in package.json.
// eslint-disable-next-line import/no-extraneous-dependencies


const coreNLP = new _stanfordCorenlp2.default.StanfordNLP({
  nlpPath: _path2.default.join(__dirname, '../../vendor/corenlp/corenlp'),
  version: '3.5.2',
  annotators: ['tokenize', 'ssplit', 'pos', 'lemma'],
  extra: {
    'tokenize.options': ['normalizeParentheses=false', 'normalizeOtherBrackets=false', 'asciiQuotes=true'].join(','),
    'ssplit.isOneSentence': 'true'
  }
});

function processWithCoreNLP(text) {
  return new Promise((resolve, reject) => {
    coreNLP.process(text, (err, result) => {
      /* istanbul ignore if */
      if (err) {
        return reject(err);
      }
      return resolve(result);
    });
  });
}

exports.default = (() => {
  var _ref2 = _asyncToGenerator(function* (text) {
    // Assuming that all sentences are separated by a line break.
    const sentences = text.split('\n');
    const lemmatized = [];

    sentences.forEach(function (sentence) {
      lemmatized.push(lemmatize(sentence));
    });

    return Promise.all(lemmatized).then(function (results) {
      return results.join('\n');
    });
  });

  return function (_x2) {
    return _ref2.apply(this, arguments);
  };
})();
//# sourceMappingURL=en.js.map