'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

/**
 * Apply transformer to every tokens extracted from a text using Mecab
 * @param {string} text a target text
 * @param {Function<Promise<string>>} transformer
 * @return {string} transformed text
 */
let transform = (() => {
  var _ref = _asyncToGenerator(function* (text, transformer) {
    const paragraphs = text.split(/\n/);

    const appliedParagraphs = paragraphs.map(function (paragraph) {
      return _asyncToGenerator(function* () {
        const sentences = paragraph.split('。');
        const appliedSentences = sentences.map(function (sentence) {
          return _asyncToGenerator(function* () {
            const parsed = yield mecab.parse(sentence);
            return parsed.map(transformer).join('');
          })();
        });

        return Promise.all(appliedSentences).then(function (results) {
          return results.join('。');
        });
      })();
    });

    return Promise.all(appliedParagraphs).then(function (results) {
      return results.join('\n');
    });
  });

  return function transform(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _xregexp = require('xregexp');

var _xregexp2 = _interopRequireDefault(_xregexp);

var _mecab = require('../mecab');

var _mecab2 = _interopRequireDefault(_mecab);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const MECAB_WORD = 0;
const mecabHome = _path2.default.join(__dirname, '..', '..', 'vendor', 'mecab');
const mecabPath = _path2.default.join(mecabHome, 'mecab', 'bin', 'mecab');
const rcPath = _path2.default.join(mecabHome, 'unidic-ojp', '.mecabrc-ojp');
const mecab = new _mecab2.default({ command: `${ mecabPath } --rcfile=${ rcPath }` });

function toVoicedChar(char) {
  const unvoiced = 'かきくけこさしすせそたちつてとはひふへほ';
  const voiced = 'がぎぐげござじずぜぞだじづでどばびぶべぼ';
  const index = unvoiced.indexOf(char);

  return index < 0 ? char : voiced[index];
}

function removeAnnotations(text) {
  /**
   * Remove a string between `left` and `right` using recursive RegExp.
   * @param {string} str a target string
   * @param {string} left left delimiter
   * @param {string} right right delimiter
   * @return {string}
   */
  const removeBetween = (str, left, right) => {
    const outsides = _xregexp2.default.matchRecursive(str, left, right, 'g', {
      valueNames: ['outside', null, null, null]
    });

    return outsides.map(outside => outside.value).join('');
  };

  // ---- Replace full-width brackets with their half-width ones.
  let _text = text.replace(/（/g, '(').replace(/）/g, ')').replace(/［/g, '[').replace(/］/g, ']').replace(/｛/g, '{').replace(/｝/g, '}');

  // --- Remove annotations.
  const brackets = [['\\(', '\\)'], ['\\[', '\\]'], ['\\{', '\\}'], ['〔', '〕'], ['【', '】']];

  brackets.forEach(pair => {
    try {
      _text = removeBetween(_text, ...pair);
    } catch (err) {
      // Fallback
      // if brackets are not properly paired,
      // try to remove the most appropriate parts of the text.
      const [left, right] = pair;

      _text = _text.replace(new RegExp(`${ left }.*?${ right }`, 'g'), '');
      _text = _text.replace(new RegExp(`(?:${ left }|${ right })`, 'g'), '');
    }
  });

  return _text;
}

exports.default = (() => {
  var _ref4 = _asyncToGenerator(function* (text) {
    // ---- Remove annotations
    const textWithoutAnnotations = removeAnnotations(text);

    const emended = textWithoutAnnotations
    // --- Replace substitutions of repeat marks with their unicode characters.
    .replace(/／〃＼/g, '〲').replace(/＼〃／/g, '〲').replace(/／＼/g, '〱').replace(/＼／/g, '〱')

    // --- Un-odorijify (single character repetition)
    .replace(/(.)ゝ/g, '$1$1').replace(/(.)ゞ/g, function (__, prev) {
      return `${ prev }${ toVoicedChar(prev) }`;
    })

    // --- Remove brackets to make a question sentence tidy.

    // Replace some half-width signs with their full-width ones.
    .replace(/｢/g, '「').replace(/｣/g, '」')

    // Remove brackets
    .replace(/(?:「|」|『|』)/g, '')

    // --- Remove full-width spaces
    // eslint-disable-next-line no-irregular-whitespace
    .replace(/　/g, '');

    // --- Un-odorijify (multi characters repetition)
    const unodorijified = yield transform(emended, function (table, i, parsed) {
      const word = table[MECAB_WORD];

      // un-odorijify (unvoiced)
      if (word.startsWith('〱')) {
        const prev = parsed[i - 1][MECAB_WORD];

        if (prev.length >= 2) {
          return word.replace(/〱/, prev);
        }
      }

      // Un-odorijify (voiced)
      if (word.startsWith('〲')) {
        const prev = parsed[i - 1][MECAB_WORD];

        if (prev.length >= 2) {
          return word.replace(/〲/, `${ toVoicedChar(prev[0]) }${ prev.substring(1) }`);
        }
      }

      return word;
    });

    // --- Wakachigaki
    // Separate each words with a half-width space (wakachigaki)
    // to enable QuizGenerator to detect word boundaries easily.
    const wakachigaki = yield transform(unodorijified, function (table) {
      return `${ table[MECAB_WORD] } `;
    });

    // --- Remove trailing spaces
    return wakachigaki.replace(/ $/mg, '');
  });

  return function (_x3) {
    return _ref4.apply(this, arguments);
  };
})();
//# sourceMappingURL=ojp.js.map