'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _regexpPresets = require('./constants/regexp-presets');

var _regexpPresets2 = _interopRequireDefault(_regexpPresets);

var _wordDivider = require('./constants/word-divider');

var _wordDivider2 = _interopRequireDefault(_wordDivider);

var _source = require('./source');

var _source2 = _interopRequireDefault(_source);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const log = (0, _debug2.default)('word-quiz-generator');

/**
 * Generate a quiz (a set of questions)
 * randomly selecting some phrases from the given material file
 * and randomly selecting question sources from the given sources.
 * NOTE: You have to run `init()` before using any other methods!
 */
class QuizGenerator {

  /**
   * @param {string} material Path to a CSV-formatted material file.
   * @param {string} sources Pathes to source text files or directries.
   * @param {string} [lang='en']
   *   IETF language tag in which the material written.
   *   `lang` is used for determining which RegExp preset should be used.
   *   If not specified or not found in the presets, the 'en' preset will be used.
   *   If you need to override each RegExps, please use individual parameters below.
   * @param {RegExp} [sentenceSeparator] Regular expression representing a sentence separator.
   * @param {RegExp} [clauseRegExp] Regular expression representing a clause.
   * @param {RegExp} [wordRegExp] Regular expression representing a word.
   * @param {RegExp} [wordBoundaryRegExp] Regular expression representing a word boundary.
   * @param {RegExp} [abbrRegExp] Regular expression representing an abbreviation mark.
   */
  constructor({
    material,
    sources,
    lang = 'en',
    sentenceSeparator,
    clauseRegExp,
    wordRegExp,
    wordBoundaryRegExp,
    abbrRegExp
  }) {
    this._material = material;
    this._sources = sources;
    this._lang = lang;
    this._sentenceSeparator = sentenceSeparator;
    this._clauseRegExp = clauseRegExp;
    this._wordRegExp = wordRegExp;
    this._wordBoundaryRegExp = wordBoundaryRegExp;
    this._abbrRegExp = abbrRegExp;
  }

  /**
   * Load and parse the files of phrases and sources.
   */
  init() {
    var _this = this;

    return _asyncToGenerator(function* () {
      _this._phrases = yield (0, _utils.parseMaterial)(_this._material);
      _this._sources = yield (0, _utils.fetchFileList)(_this._sources, /\.txt$/);
      _this._sources = _this._sources.map(function (path) {
        return new _source2.default(path);
      });
      _this._loadRegExpPreset();
    })();
  }

  _loadRegExpPreset() {
    const preset = _regexpPresets2.default[this._lang] || _regexpPresets2.default.en;

    this._sentenceSeparator = this._sentenceSeparator || preset.sentenceSeparator;
    this._clauseRegExp = this._clauseRegExp || preset.clauseRegExp;
    this._wordRegExp = this._wordRegExp || preset.wordRegExp;
    this._wordBoundaryRegExp = this._wordBoundaryRegExp || preset.wordBoundaryRegExp;
    this._abbrRegExp = this._abbrRegExp || preset.abbrRegExp;
  }

  /**
   * Generate a quiz.
   * @param {string} sections "-" separated string
   *                          representing a scope of the quiz.
   *                          e.g. 5-10
   * @param {number} size # of questions in a quiz to generate.
   * @return {Array<Question>} Generated quiz.
   */
  quiz({ sections, size }) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const [min = 0, max] = sections.split('-').map(function (num) {
        return Number(num);
      });
      const phrasesToTest = _lodash2.default.shuffle(_this2._phrases.filter(function (phrase) {
        return _lodash2.default.inRange(phrase.section, min, (max || min) + 1);
      }));
      const quiz = [];

      const chunks = _lodash2.default.chunk(phrasesToTest, size);

      yield new Promise(function (resolve) {
        chunks.reduce(function (promise, chunk) {
          return promise.then(function () {
            if (quiz.length >= size) {
              return Promise.resolve();
            }

            return Promise.all(chunk.map(function (phrase) {
              return _asyncToGenerator(function* () {
                const question = yield _this2.question(phrase);

                if (question) {
                  quiz.push(question);
                }

                if (quiz.length >= size) {
                  resolve();
                }
              })();
            }));
          });
        }, Promise.resolve()).then(function () {
          return resolve();
        });
      });

      return _lodash2.default.take(quiz, size);
    })();
  }

  /**
   * Generate a question using the given phrase.
   * @param {{phrase, answer, reference}} phrase
   * @return {Question|null}
   */
  question(phrase) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const sources = _lodash2.default.shuffle(_this3._sources);

      return new Promise(function (resolve) {
        Promise.all(sources.map(function (src) {
          return _asyncToGenerator(function* () {
            const question = yield _this3.generateQuestionFromSource({ phrase, src });

            if (question) {
              resolve(question);
            }
          })();
        })).then(function () {
          return resolve(null);
        });
      });
    })();
  }

  /**
   * Generate a question using the given phrase and source.
   * @param {{phrase, answer, reference}} phrase
   * @param {Source} src
   * @return {Question|null}
   */
  generateQuestionFromSource({ phrase, src }) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      const result = yield _this4.selectSentence({ phrase: phrase.phrase, src });

      if (!result) {
        return null;
      }

      const { sentenceIndex, wordIndexes } = result;
      const text = yield src.preprocessed.getText();
      const reference = yield src.preprocessed.getReference();
      const sentence = _this4.getSentenceAt({ index: sentenceIndex, text });

      /* istanbul ignore if */
      if (!sentence) {
        return null;
      }

      const body = _this4.parseQuestionSentence({ sentence, wordIndexes });

      return {
        body,
        answer: phrase.answer,
        reference,
        phrase: phrase.phrase
      };
    })();
  }

  /**
   * @param {string} original (non-lemmatized) sentence
   * @param {Array<number>} wordIndexes index numbers representing which word is a part of question.
   * @return {Array<{ text: string, isQuestionPart: bool, isMark: bool, isDivider: bool }>}
   */
  parseQuestionSentence({ sentence, wordIndexes }) {
    // Here we assume word divider is space.
    // If you are dealing with a language it is not the case (e.g. Japanese),
    // firstly you have to convert a text using a preprocessor.
    const tokenRegExp = new RegExp(`(${ this._wordRegExp.source })(\\s*)`, 'g');
    const tokens = [];
    const divider = _lodash2.default.findKey(_wordDivider2.default, regs => regs.some(reg => reg.test(this._lang)));
    let currentWordIndex = 0;
    let prevLastIndex = 0;
    let matched;

    // eslint-disable-next-line no-cond-assign
    while (matched = tokenRegExp.exec(sentence)) {
      const [, token, hasDivider] = matched;
      const punctuation = sentence.substring(prevLastIndex, matched.index);
      const isQuestionWord = wordIndexes[0] === currentWordIndex;
      const isSuccessive = isQuestionWord && wordIndexes[0] + 1 === wordIndexes[1];

      if (punctuation) {
        tokens.push({
          text: punctuation,
          isMark: true
        });
      }

      tokens.push({
        text: token,
        isQuestionPart: isQuestionWord
      });

      if (hasDivider && divider) {
        tokens.push({
          text: divider,
          isQuestionPart: isSuccessive,
          isMark: true,
          isDivider: true
        });
      }

      if (isQuestionWord) {
        wordIndexes.shift();
      }

      currentWordIndex += 1;
      prevLastIndex = tokenRegExp.lastIndex;
    }

    const stopPunctuation = sentence.substring(prevLastIndex);
    tokens.push({
      text: stopPunctuation,
      isMark: true
    });

    log({ tokens });

    return tokens;
  }

  /**
   * Generate a question from a given phrase and source.
   * @param {string} phrase phrase to find.
   * @param {Source} src
   * @return {{ sentenceIndex: number, wordIndexes: Array<number> }}
   */
  selectSentence({ phrase, src }) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      const result = yield _this5.findSentenceIncludingPhrase({ phrase, src });

      if (!result) {
        return null;
      }

      const {
        lemmatizedSentence,
        index: sentenceIndex
      } = result;
      const wordIndexes = _this5.findWordIndexesWithinLemmatizedSentence({
        lemmatizedSentence,
        phrase
      });

      log({
        phrase,
        src: _lodash2.default.last(src.path.split('/')),
        sentenceIndex,
        lemmatizedSentence,
        wordIndexes
      });

      return { sentenceIndex, wordIndexes };
    })();
  }

  findWordIndexesWithinLemmatizedSentence({ lemmatizedSentence, phrase }) {
    const sentence = lemmatizedSentence;
    const phraseRegExp = this.getPhraseRegExp(phrase);
    const wordIndexes = [];

    sentence.replace(phraseRegExp, (matched, block, offset) => {
      const left = sentence.substring(0, offset);
      const indexOffset = (left.match(this._wordRegExp) || []).length;
      const words = _lodash2.default.flatten(phrase.split('|').map(exp => exp.replace(this._abbrRegExp, '').match(this._wordRegExp) || []));
      let wordsCount = words.length;
      const escapedWords = words.map(word => word.replace(/\W/g, '\\$&')).join('|');
      const wordsRegExp = new RegExp(`(?:${ escapedWords })`, 'gi');

      block.replace(wordsRegExp, (word, offset2) => {
        if (wordsCount <= 0) {
          return;
        }

        const inlineLeft = block.substring(0, offset2);
        const inlineIndex = (inlineLeft.match(this._wordRegExp) || []).length;

        wordIndexes.push(indexOffset + inlineIndex);
        wordsCount -= 1;
      });
    });

    return wordIndexes;
  }

  findSentenceIncludingPhrase({ src, phrase }) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      const text = yield src.lemmatized.getText();
      const phraseRegExp = _this6.getPhraseRegExp(phrase);
      const matches = [];

      // Find the phrase in the source text
      // and store the match information.
      // We use 'replace' here because nether RegExp#exec nor String#match
      // have a detail of each matches like a matched block and an offset etc.
      text.replace(phraseRegExp, function (matched, block, offset) {
        const lastIndex = offset + matched.length;
        matches.push({ matched, block, offset, lastIndex });
      });

      if (!matches.length) {
        return null;
      }

      log({
        phraseRegExp,
        src: _lodash2.default.last(src.path.split('/')),
        matches
      });

      const selected = _lodash2.default.sample(matches);
      const leftText = text.substring(0, selected.offset);
      const leftSentences = leftText.match(_this6._sentenceSeparator) || [];
      const leftContext = _lodash2.default.last(leftText.split(_this6._sentenceSeparator));
      const rightContext = _lodash2.default.head(text.substring(selected.lastIndex).split(_this6._sentenceSeparator));
      const lemmatizedSentence = leftContext + selected.matched + rightContext;
      const index = leftSentences.length;

      return { lemmatizedSentence, index };
    })();
  }

  getPhraseRegExp(phrase) {
    const phraseWithAbbrRegExp = phrase.replace(this._abbrRegExp, `?(?:${ this._clauseRegExp.source })?`);
    const wordBoundaryRegExp = this._wordBoundaryRegExp.source;
    const phraseRegExp = new RegExp(`(?:^|${ wordBoundaryRegExp })(${ phraseWithAbbrRegExp })${ wordBoundaryRegExp }`, 'gim');

    return phraseRegExp;
  }

  /**
   * Get the 'index'-th sentence of the given text.
   * @param {number} index
   * @param {string} text
   * @return {string}
   */
  getSentenceAt({ index, text }) {
    const body = text.split(this._sentenceSeparator)[index];

    /* istanbul ignore if */
    if (!body) {
      return null;
    }

    const rightContext = (text.split(body)[1] || '').substring(0, 5).trim();
    if (!rightContext) {
      return body;
    }
    const endMarks = rightContext.match(this._sentenceSeparator);
    if (!endMarks) {
      return body;
    }
    const endMark = endMarks[0].trim();

    return body + endMark;
  }
}
exports.default = QuizGenerator;
//# sourceMappingURL=quiz-generator.js.map