'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fsExtraPromise = require('fs-extra-promise');

var _fsExtraPromise2 = _interopRequireDefault(_fsExtraPromise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class Source {

  /**
   * @param {string} path Path to a source file.
   * @param {boolean} shouldEnsureFile Whether or not create a file if not exists.
   */
  constructor(path, shouldEnsureFile) {
    if (shouldEnsureFile) {
      _fsExtraPromise2.default.ensureFileSync(path);
    } else {
      _fsExtraPromise2.default.statSync(path);
    }

    this.path = path;
  }

  get preprocessed() {
    if (this._preprocessed) {
      return this._preprocessed;
    }

    try {
      return this._preprocessed = new Source(`${ this.path }.preprocessed`);
    } catch (err) {
      return this._preprocessed = this;
    }
  }

  get lemmatized() {
    if (this._lemmatized) {
      return this._lemmatized;
    }

    try {
      return this._lemmatized = new Source(`${ this.path }.lemmatized`);
    } catch (err) {
      return this._lemmatized = this;
    }
  }

  /**
   * Read the content of the file, parse it, and populate the results.
   */
  _fetchContent() {
    var _this = this;

    return _asyncToGenerator(function* () {
      const content = yield _fsExtraPromise2.default.readFileAsync(_this.path, 'utf-8');
      const [reference, ...text] = content.split(/[\n\r]/);

      _this.setReference(reference);
      _this.setText(text.join('\n'));
    })();
  }

  /**
   * Return the reference information of this source.
   * @return {string}
   */
  getReference() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      if (_this2._reference) {
        return _this2._reference;
      }

      yield _this2._fetchContent();
      return _this2._reference;
    })();
  }

  /**
   * Set the reference information of this source.
   * @param {string} reference
   */
  setReference(reference) {
    this._reference = reference;
  }

  /**
   * Return the body text of this source.
   * @return {string}
   */
  getText() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      if (_this3._text) {
        return _this3._text;
      }

      yield _this3._fetchContent();
      return _this3._text;
    })();
  }

  /**
   * Set the body text of this source.
   * @param {string} text
   */
  setText(text) {
    this._text = text;
  }

  /**
   * Write the reference and the body text to the file.
   */
  save() {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      const reference = yield _this4.getReference();
      const text = yield _this4.getText();
      yield _fsExtraPromise2.default.writeFileAsync(_this4.path, `${ reference }\n${ text }`);
    })();
  }

  /**
   * Preprocess the text using the given preprocessor and save the result.
   * @param {Function<Promise<string>>} preprocessor
   */
  preprocess(preprocessor) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      const source = new Source(`${ _this5.path }.preprocessed`, true);
      const reference = yield _this5.getReference();
      const text = yield _this5.getText();
      const preprocessed = yield preprocessor(text);

      source.setReference(reference);
      source.setText(preprocessed);
      yield source.save();
    })();
  }

  /**
   * Lemmatize the preprocessed text using the given lemmatizer
   * and save the result.
   * @param {Function<Promise<string>>} lemmatizer
   */
  lemmatize(lemmatizer) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      const source = new Source(`${ _this6.path }.lemmatized`, true);
      const reference = yield _this6.preprocessed.getReference();
      const text = yield _this6.preprocessed.getText();
      const lemmatized = yield lemmatizer(text);

      source.setReference(reference);
      source.setText(lemmatized);
      yield source.save();
    })();
  }
}
exports.default = Source;
//# sourceMappingURL=source.js.map